package android.rdvmedecins.dao;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.http.converter.json.GsonHttpMessageConverter;
import org.springframework.web.client.RestTemplate;

import android.rdvmedecins.entities.exceptions.RdvMedecinsException;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

public class Dao implements IDao {

	// client REST
	private RestTemplate restTemplate;
	// mapper JSON
	private Gson gson = new Gson();
	// délai d'attente maximal
	private int timeout;

	// constructeur
	public Dao() {
		// on crée un objet [RestTemplate]
		restTemplate = new RestTemplate();
		// on le configure
		restTemplate.getMessageConverters().add(new GsonHttpMessageConverter());
		restTemplate.getMessageConverters().add(new StringHttpMessageConverter());
	}

	// exécution de l'appel au service REST
	public String executeRestService(String method, String urlService, Object request, Map<String, String> paramètres) {

		// on vérifie que le serveur distant répond assez vite
		// une exception est lancée sinon
		checkResponsiveness(urlService);
		// vérification méthode HTTP
		method = method.toLowerCase();
		if (!method.equals("get") && !method.equals("post")) {
			throw new RdvMedecinsException(String.format("L'argument [method] doit avoir la valeur post ou get dans [%s]",
					urlService));
		}
		try {
			// exécution service
			String réponse = null;
			if (method.equals("get")) {
				réponse = restTemplate.getForObject(urlService, String.class, paramètres);
			} else {
				réponse = restTemplate.postForObject(urlService, request, String.class, paramètres);
			}
			// résultat
			Map<String, Object> map = gson.fromJson(réponse, new TypeToken<Map<String, Object>>() {
			}.getType());
			// erreur ?
			String erreur = (String) map.get("error");
			if (erreur != null) {
				throw new RdvMedecinsException(String.format("Le serveur a renvoyé l'erreur suivante : %s",
						(String) map.get("message")), 90);
			}
			// data ?
			Object data = map.get("data");
			if (data == null) {
				throw new RdvMedecinsException(String.format("Réponse incorrecte du serveur : %s", réponse), 91);
			}
			// c'est bon - on rend le résultat
			return gson.toJson(data);
		} catch (Exception ex) {
			throw new RdvMedecinsException(String.format("Une erreur s'est produite : %s", ex.getMessage()), 92);
		}
	}

	private void checkResponsiveness(String urlService) {
		// on crée l'URI du service distant
		String url = urlService.replace("{", "").replace("}", "");
		URI service = null;
		try {
			service = new URI(url);
		} catch (URISyntaxException ex) {
			throw new RdvMedecinsException(String.format("Format d'URL incorrect [%s]", urlService), ex, 93);
		}
		// on se connecte au service
		Socket client = null;
		try {
			// avec attente maximale définie par configuration
			client = new Socket();
			client.connect(new InetSocketAddress(service.getHost(), service.getPort()), timeout);
		} catch (Throwable th) {
			// le serveur n'a pas répondu
			throw new RdvMedecinsException(String.format("Le service distant n'a pas répondu assez vite : %s",
					th.getMessage()), 94);
		} finally {
			// on libère les ressources
			if (client != null) {
				try {
					client.close();
				} catch (IOException ex) {
					Logger.getLogger(Dao.class.getName()).log(Level.SEVERE, null, ex);
				}
			}
		}

	}

	// délai d'attente maximal
	public void setTimeout(int millis) {
		this.timeout = millis;
	}

}
